"""Altissimo API Class file."""
import datetime

import jwt

from flask import abort
from flask import request


class API:
    """Altissimo API Class."""

    def __init__(self, auth_key=None):
        """Initialize an API object."""
        self.auth_header = None
        self.auth_key = auth_key
        self.auth_token = None
        self.auth_token_info = None

    def authorize_request(self):
        """Authorize an API request."""
        print("Checking Auth Header...")
        self.check_auth_header()
        print("Checking Auth Header Bearer...")
        self.check_auth_header_bearer()
        print("Checking Auth Token...")
        self.check_auth_token()
        print("Validating Auth Token...")
        self.validate_auth_token()
        return True

    def check_auth_header(self):
        """Check the auth header."""
        auth_header = request.headers.get("Authorization")
        if not auth_header:
            abort(401, "Required Authorization header not found.")
            return None
        self.auth_header = auth_header
        return auth_header

    def check_auth_header_bearer(self):
        """Check the auth header bearer."""
        if not self.auth_header.lower().startswith("bearer "):
            abort(401, "Invalid Authorization header. Bearer type required.")
            return None
        return True

    def check_auth_token(self):
        """Check the auth token."""
        auth_token = self.auth_header.split(" ")[1]
        if not auth_token:
            abort(401, "Required Bearer token not found.")
            return None
        self.auth_token = auth_token
        return auth_token

    def create_jwt(self, data, aud=None, iss=None, seconds=3600, sub=None):
        """Create a JWT."""
        payload = {
            "aud": aud,
            "exp": datetime.datetime.now() + datetime.timedelta(seconds=seconds),
            "iat": datetime.datetime.now(),
            "iss": iss,
            "nbf": datetime.datetime.now(),
            "sub": sub,
            "data": data,
        }
        return jwt.encode(payload, self.auth_key, algorithm="HS256")

    def validate_auth_token(
        self,
        algorithms=None,
        options=None,
        audience=None,
        issuer=None,
        leeway=None,
    ):
        """Validate the auth token (JWT)."""
        if not algorithms:
            algorithms = ["HS256"]
        if not leeway:
            leeway = 10
        if not options:
            options = {
                "require": ["exp", "iat", "nbf"],
                "verify_aud": False,
                "verify_exp": True,
                "verify_iat": True,
                "verify_iss": False,
                "verify_nbf": True,
            }

        try:
            payload = jwt.decode(
                self.auth_token,
                self.auth_key,
                algorithms=algorithms,
                options=options,
                audience=audience,
                issuer=issuer,
                leeway=leeway,
            )
        except jwt.exceptions.ExpiredSignatureError:
            abort(401, "Token expired.")
            return None
        except jwt.exceptions.InvalidAudienceError:
            abort(401, "Invalid audience.")
            return None
        except jwt.exceptions.InvalidIssuerError:
            abort(401, "Invalid issuer.")
            return None
        except jwt.exceptions.MissingRequiredClaimError as e:
            abort(401, f"Missing required claim. [{e}]")
            return None
        except jwt.exceptions.InvalidTokenError:
            abort(401, "Invalid token.")
            return None

        if not payload:
            abort(401, "Invalid token. Missing payload.")
            return None

        data = payload.get("data", {})
        if not data:
            abort(401, "Invalid token. Missing data.")
            return None

        self.auth_token_info = payload

        return True



# -*- coding: utf-8 -*-
"""Altissimo Firestore class file."""
from typing import Dict
from typing import List

from google.cloud import firestore
from google.cloud import firestore_v1
from google.cloud.firestore import DocumentReference
from google.cloud.firestore import DocumentSnapshot
from google.cloud.firestore_v1 import aggregation

from .fastapi import Pagination
from .fastapi import PaginatedList
from .tools import chunks

# pylint: disable=line-too-long
# pylint: disable=too-many-arguments
# pylint: disable=too-many-instance-attributes


class FirestoreCollection:
    """Altissimo Firestore Collection class."""

    def __init__(self, db, collection_name: str):
        """Initialize Collection class."""
        self.collection = db.collection(collection_name)
        self.db = db
        self.name = collection_name

    @classmethod
    def count(cls, ref: DocumentReference):
        """Return the number of results in a query reference."""
        for result in aggregation.AggregationQuery(ref).count().get():
            return result[0].value
        return 0

    def create_document(self, doc_id: str, data: dict, remove_id: bool = True) -> dict:
        """Create a document in this collection."""
        ref = self.collection.document(doc_id)
        if remove_id and "id" in data:
            del data["id"]
        ref.create(data)
        doc = ref.get()
        return {"id": doc.id, **doc.to_dict()}

    def delete_document(self, doc_id: str) -> dict:
        """Delete a document in this collection."""
        self.collection.document(doc_id).delete()
        return {"id": doc_id}

    def get_document(self, doc_id: str) -> DocumentSnapshot:
        """Return the document from this collection."""
        return self.collection.document(doc_id).get()

    def get_document_dict(self, doc_id: str, include_id: bool = True) -> dict:
        """Return a document from this collection as a dict."""
        doc = self.get_document(doc_id)
        if include_id:
            return {"id": doc.id, **doc.to_dict()}
        return doc.to_dict()

    def list_documents(self) -> List[DocumentSnapshot]:
        """Return a list of documents from this collection."""
        return list(self.collection.stream())
    get_documents = list_documents

    def list_documents_dict(self) -> Dict[str, DocumentSnapshot]:
        """Return a dict of dicts from this collection."""
        return {x.id: x for x in self.list_documents()}
    get_documents_dict = list_documents_dict

    def list_dicts(self, include_id: bool = True) -> List[dict]:
        """Return a documents from this collection as a list of dicts."""
        if include_id:
            return [{"id": x.id, **x.to_dict()} for x in self.list_documents()]
        return [x.to_dict() for x in self.list_documents()]
    get_dicts = list_dicts

    def list_dicts_dict(self, include_id: bool = True) -> Dict[str, dict]:
        """Return documents from this collection as a dict of dicts."""
        return {x["id"]: x for x in self.list_dicts(include_id)}
    get_dicts_dict = list_dicts_dict

    def __next_page_query(self, pagination) -> List[dict]:
        """Return results from a next_page cursor query."""
        snapshot = self.collection.document(pagination.next_cursor).get()
        if not snapshot:
            return self.__default_page_query(pagination)
        direction = firestore.Query.ASCENDING
        if pagination.direction == "descending":
            direction = firestore.Query.DESCENDING

        # create collection query with direction
        ref = self.collection.order_by(pagination.order_by, direction=direction)

        # get next page of results
        ref = ref.start_after(snapshot).limit(pagination.limit + 1)

        # retrieve items from stream
        items = []
        for doc in ref.stream():
            item = doc.to_dict()
            item["id"] = doc.id
            items.append(item)
        return items

    def __previous_page_query(self, pagination) -> List[dict]:
        """Return results from a previous_page cursor query."""
        snapshot = self.collection.document(pagination.previous_cursor).get()
        if not snapshot:
            return self.__default_page_query(pagination)
        direction = firestore.Query.DESCENDING
        if pagination.direction == "descending":
            direction = firestore.Query.ASCENDING

        # create collection query with direction
        ref = self.collection.order_by(pagination.order_by, direction=direction)

        # get next page of results
        ref = ref.start_after(snapshot).limit(pagination.limit + 1)

        # retrieve items from stream and return in reverse order
        items = []
        for doc in ref.stream():
            item = doc.to_dict()
            item["id"] = doc.id
            items.append(item)
        items.reverse()
        return items

    def __default_page_query(self, pagination) -> List[dict]:
        """Return results from pagination query with no cursor."""
        # get sort direction
        direction = firestore.Query.ASCENDING
        if pagination.direction == "descending":
            direction = firestore.Query.DESCENDING
        ref = self.collection.order_by(pagination.order_by, direction=direction)

        # check for limit
        if pagination.limit:
            ref = ref.limit(pagination.limit + 1)

        # retrieve items from stream
        items = []
        for doc in ref.stream():
            item = doc.to_dict()
            item["id"] = doc.id
            items.append(item)
        return items

    def list_dicts_paginated(self, pagination: Pagination) -> PaginatedList:
        """Return an embedded list of documents from this collection with pagination."""
        # ref = self.collection

        next_cursor = ""
        previous_cursor = ""

        if pagination.next_cursor:
            items = self.__next_page_query(pagination)
            previous_cursor = items[0]["id"]
            if len(items) > pagination.limit:
                items = items[:-1]
                next_cursor = items[-1]["id"]

        elif pagination.previous_cursor:
            items = self.__previous_page_query(pagination)
            next_cursor = items[-1]["id"]
            if len(items) > pagination.limit:
                items = items[1:]
                previous_cursor = items[0]["id"]

        else:
            items = self.__default_page_query(pagination)
            if len(items) > pagination.limit:
                items = items[:-1]
                next_cursor = items[-1]["id"]

        total = None
        if pagination.include_total:
            total = self.count(self.collection)

        if total:
            return PaginatedList(items=items, next_cursor=next_cursor, previous_cursor=previous_cursor, total=total)
        return PaginatedList(items=items, next_cursor=next_cursor, previous_cursor=previous_cursor)

    def save_document(self, doc_id: str, data: dict, remove_id: bool = True) -> dict:
        """Save a document in this collection."""
        ref = self.collection.document(doc_id)
        if remove_id and "id" in data:
            del data["id"]
        ref.set(data)
        doc = ref.get()
        return {"id": doc.id, **doc.to_dict()}

    def update_document(self, doc_id: str, data: dict, remove_id: bool = True) -> dict:
        """Update a document in this collection."""
        ref = self.collection.document(doc_id)
        if remove_id and "id" in data:
            del data["id"]
        ref.update(data)
        doc = ref.get()
        return {"id": doc.id, **doc.to_dict()}


class FirestoreUpdate:
    """Altissimo Firestore Update class."""

    def __init__(self, c, data, delete_items=False, diff=False, key_name=None):
        """Initialize a Firestore Update instance."""
        self.c = c
        self.collection = c.name

        self.db = self.c.db

        self.data = data
        self.delete_items = delete_items
        self.diff = diff
        self.key_name = key_name

        self.update(data)

    @classmethod
    def dict_keys(cls, *args: dict):
        """Return a list of keys from one or more dicts."""
        keys = set()
        for data in args:
            keys.update(set(data.keys()))
        return list(keys)

    @classmethod
    def keys_to_str(cls, data: dict) -> dict:
        """Return a dict with all the keys converted to string format."""
        items = {}
        for key, value in data.items():
            items[str(key)] = value
        return items

    @classmethod
    def remap_dict(cls, data: dict, key_name: str) -> Dict[str, dict]:
        """Return a dictionary remapped with a different field as the key."""
        response = {}
        for item in data.values():
            key = item[key_name]
            response[key] = item
        return response

    def __diff_items(self, item_a, item_b):
        """Diff two dict items and return the differences."""
        output = []
        for k in sorted(set(list(item_a) + list(item_b))):
            if self.key_name and k == "id":
                continue
            if k not in item_a:
                output.append(f"  {k}: {item_b[k]} (added)")
                continue
            if k not in item_b:
                output.append(f"  {k}: {item_a[k]} (removed)")
                continue
            a = item_a.get(k)
            b = item_b.get(k)
            if a != b:
                output.append(f"  {k}: {a} -> {b}")
        return output

    def __prepare_adds(self, current, data) -> Dict[str, dict]:
        """Return a dict of records to add to firestore."""
        adds: Dict[str, dict] = {}
        for key, item in data.items():
            if key not in current:
                doc_id = key
                if self.key_name:
                    doc_id = self.c.collection.document().id
                adds[doc_id] = item
        return adds

    def __prepare_deletes(self, current, data) -> List[str]:
        """Return a list of document IDs to delete from Firestore."""
        deletes: List[str] = []
        for key, item in current.items():
            if key not in data:
                doc_id = key
                if self.key_name:
                    doc_id = item["id"]
                deletes.append(doc_id)
        return deletes

    def __prepare_updates(self, current, data) -> Dict[str, dict]:
        """Return a dict of records to update in Firestore."""
        updates: Dict[str, dict] = {}
        for key, item in data.items():
            if key not in current:
                continue
            c = current[key]
            if c == item:
                continue
            output = self.__diff_items(c, item)
            if not output:
                continue
            if self.diff and output:
                print(f"Updating {key}:")
                print("\n".join(output))
            doc_id = key
            if self.key_name:
                doc_id = c["id"]
            updates[doc_id] = item
        return updates

    def __run_batch_adds(self, adds: Dict[str, dict]) -> None:
        """Perform the adds to Firestore."""
        for chunk in chunks(list(adds), 500):
            batch = self.db.batch()
            for doc_id in chunk:
                item = adds[doc_id]
                ref = self.c.collection.document(doc_id)
                batch.set(ref, item)
            batch.commit()
            print(f"Added {len(chunk)} docs to {self.collection}")

    def __run_batch_deletes(self, deletes: List[str]) -> None:
        """Perform the adds to Firestore."""
        if not self.delete_items:
            return
        for chunk in chunks(list(deletes), 500):
            batch = self.db.batch()
            for doc_id in chunk:
                ref = self.c.collection.document(doc_id)
                batch.delete(ref)
            batch.commit()
            print(f"Deleted {len(chunk)} docs from {self.collection}")

    def __run_batch_updates(self, updates: Dict[str, dict]):
        """Perform the adds to Firestore."""
        # do batch updates
        for chunk in chunks(list(updates), 500):
            batch = self.db.batch()
            for doc_id in chunk:
                item = updates[doc_id]
                ref = self.c.collection.document(doc_id)
                batch.set(ref, item)
            batch.commit()
            print(f"Updated {len(chunk)} docs in {self.collection}")

    def prepare_data(self, current, data):
        """Prepare the adds, deletes, and updates."""
        adds = self.__prepare_adds(current, data)
        deletes = self.__prepare_deletes(current, data)
        updates = self.__prepare_updates(current, data)
        print(
            f"[{self.collection}]: "
            f"Adds: {len(adds)}, "
            f"Deletes: {len(deletes)}, "
            f"Updates: {len(updates)}."
        )
        return adds, deletes, updates

    def update(self, data):
        """Run the Update process."""
        current = self.c.list_dicts_dict()

        if self.key_name:
            current = self.remap_dict(current, self.key_name)
            data = self.remap_dict(data, self.key_name)

        data = self.keys_to_str(data)

        adds, deletes, updates = self.prepare_data(current, data)

        self.__run_batch_adds(adds)
        self.__run_batch_deletes(deletes)
        self.__run_batch_updates(updates)

        print(f"[{self.collection}]: Done.")


class Firestore:
    """Altissimo Firestore class."""

    def __init__(self, project=None, credentials=None, database=None):
        """Initialize Firestore class."""
        self.project = project
        self.credentials = credentials
        self.database = database

        self.db = firestore.Client(
            project=project,
            credentials=credentials,
            database=database,
        )
        self.firestore = firestore
        self.firestore_v1 = firestore_v1

    #
    # Collections
    #
    def collection(self, collection_name) -> FirestoreCollection:
        """Return a Collection instance."""
        return FirestoreCollection(self.db, collection_name)

    def create_collection_document(self, collection, doc_id, data, remove_id=True) -> dict:
        """Create the doc in a collection."""
        c = self.collection(collection)
        return c.create_document(doc_id, data, remove_id)
    create_document = create_collection_document

    def delete_collection_document(self, collection, doc_id) -> dict:
        """Delete a document from a collection."""
        return self.collection(collection).delete_document(doc_id)
    delete_document = delete_collection_document

    def get_collection_document(self, collection, doc_id) -> dict:
        """Return the doc in a collection."""
        return self.collection(collection).get_document(doc_id)
    get_doc = get_collection_document
    get_document = get_collection_document

    def get_collection_document_dict(self, collection, doc_id, include_id: bool = True) -> dict:
        """Return a doc from collection as a dictionary."""
        return self.collection(collection).get_document_dict(doc_id, include_id)
    get_doc_dict = get_collection_document_dict
    get_document_dict = get_collection_document_dict

    def list_collection_dicts(self, collection, include_id=True) -> list:
        """Return a list of dicts from a collection."""
        return self.collection(collection).list_dicts(include_id)
    get_collection = list_collection_dicts
    get_collection_dicts = list_collection_dicts
    get_collection_items = list_collection_dicts

    def list_collection_dicts_dict(self, collection, include_id=True) -> Dict[str, dict]:
        """Return a list of dicts from a collection."""
        return self.collection(collection).list_dicts_dict(include_id)
    get_collection_dict = list_collection_dicts_dict
    get_collection_dicts_dict = list_collection_dicts_dict
    get_collection_items_dict = list_collection_dicts_dict

    def list_collection_dicts_paginated(self, collection, pagination) -> PaginatedList:
        """Return a paginated list of dicts from a Firestore collection."""
        return self.collection(collection).list_dicts_paginated(pagination)
    get_collection_dicts_paginated = list_collection_dicts_paginated

    def list_collection_documents(self, collection) -> List[DocumentSnapshot]:
        """Return a list of docs in a collection."""
        return self.collection(collection).list_documents()
    get_docs = list_collection_documents
    get_documents = list_collection_documents
    get_collection_docs = list_collection_documents

    def list_collection_documents_dict(self, collection) -> Dict[str, DocumentSnapshot]:
        """Return documentss from a collection as a dict."""
        return self.collection(collection).list_documents_dict()
    get_docs_dict = list_collection_documents_dict
    get_documents_dict = list_collection_documents_dict
    get_collection_docs_dict = list_collection_documents_dict

    def save_collection_document(
        self,
        collection: str,
        doc_id: str,
        data: dict,
        remove_id: bool = True
    ) -> dict:
        """Save a document in a collection."""
        return self.collection(collection).save_document(doc_id, data, remove_id=remove_id)
    save_document = save_collection_document

    def update_collection_document(
        self,
        collection: str,
        doc_id: str,
        data: dict,
        remove_id: bool = True
    ) -> dict:
        """Update a document in a collection."""
        return self.collection(collection).update_document(doc_id, data, remove_id=remove_id)
    update_document = update_collection_document

    #
    # Collection Groups
    #
    def get_collection_group(self, collection, include_id=True) -> list:
        """Return a dict of dicts from a collection group."""
        ref = self.db.collection_group(collection)
        items = []
        for doc in ref.stream():
            item = doc.to_dict()
            if include_id:
                item["id"] = doc.id
            items.append(item)
        return items

    def get_collection_group_dict(self, collection, include_id=True) -> dict:
        """Return a dict of dicts from a collection group."""
        ref = self.db.collection_group(collection)
        items = {}
        for doc in ref.stream():
            item = doc.to_dict()
            if include_id:
                item["id"] = doc.id
            items[doc.id] = item
        return items

    #
    # Update
    #
    def update(self, collection_name, data, delete_items=False, diff=False, key_name=None) -> FirestoreUpdate:
        """Return a Firestore Update instance."""
        c = self.collection(collection_name)
        return FirestoreUpdate(c, data, delete_items, diff, key_name)
    update_collection = update



"""Altissimo Tools Tests file."""
import types

from .tools import chunks
from .tools import list_to_dict

fake_list = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]

fake_dict_list = [
    {"id": 1, "name": "one", "key": "one", "value": 1},
    {"id": 2, "name": "two", "key": "two", "value": 1},
    {"id": 3, "name": "three", "key": "three", "value": 1},
    {"id": 4, "name": "four", "key": "four", "value": 1},
    {"id": 5, "name": "five", "key": "five", "value": 1},
    {"id": 6, "name": "six", "key": "six", "value": 2},
    {"id": 7, "name": "seven", "key": 0, "value": 2},
    {"id": 8, "name": "eight", "key": "", "value": 2},
    {"id": 9, "name": "nine", "key": None, "value": 2},
    {"id": 10, "name": "ten", "value": 2},
]


def test_chunks_generator():
    """Test that chunks returns a generator."""
    assert isinstance(chunks(fake_list, 4), types.GeneratorType)


def test_chunks_list_length():
    """Test that chunks returns the correct number of chunks."""
    assert len(list(chunks(fake_list, 4))) == 5


def test_chunks_list():
    """Test that chunks returns the correct chunks."""
    assert list(chunks(fake_list, 3)) == [
        [1, 2, 3],
        [4, 5, 6],
        [7, 8, 9],
        [10, 11, 12],
        [13, 14, 15],
        [16, 17, 18],
        [19, 20]
    ]
    assert list(chunks(fake_list, 4)) == [
        [1, 2, 3, 4],
        [5, 6, 7, 8],
        [9, 10, 11, 12],
        [13, 14, 15, 16],
        [17, 18, 19, 20]
    ]
    assert list(chunks(fake_list, 5)) == [
        [1, 2, 3, 4, 5],
        [6, 7, 8, 9, 10],
        [11, 12, 13, 14, 15],
        [16, 17, 18, 19, 20]
    ]


def test_list_to_dict_dict():
    """Test that list_to_dict returns a dictionary."""
    assert isinstance(list_to_dict(fake_dict_list), dict)


def test_list_to_dict_length():
    """Test that list_to_dict returns the correct number of items."""
    assert len(list_to_dict(fake_dict_list)) == len(fake_dict_list)


def test_list_to_dict_duplicate_keys():
    """Test that list_to_dict properly handles duplicate keys."""
    assert len(list_to_dict(fake_dict_list, key="value")) == 2


def test_list_to_dict_false_keys():
    """Test that list_to_dict properly handles keys that evaluate to false."""
    assert len(list_to_dict(fake_dict_list, key="key")) == 8



"""Altissimo Tools moduule."""


def chunks(items, n):
    """Yield successive chunks of size n from list items."""
    for i in range(0, len(items), n):
        yield items[i:i + n]


def list_to_dict(items, key="id"):
    """Convert a list of dicts to a dict of dicts by key."""
    results = {}
    for item in items:
        k = item.get(key)
        if k is None:
            continue  # Skip items with null key
        if k in results:
            continue  # Skip items with duplicate key
        results[k] = item
    return results



# -*- coding: utf-8 -*-
"""Altissimo FastAPI class file."""
import json

from enum import Enum
from typing import List
from typing import Optional

from pydantic import BaseModel
# from pydantic import Field
from pydantic import model_validator

from fastapi import Query
from fastapi import Request
from fastapi import Response
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import StreamingResponse

# pylint: disable=too-few-public-methods


class JSONPrettyPrintMiddleware(BaseHTTPMiddleware):
    """Fast API JSON Pretty Prinut Middleware."""

    def __init__(self, app, excluded_paths: List[str] | None = None):
        super().__init__(app)
        self.excluded_paths = excluded_paths or ["/docs", "/redoc", "/openapi.json"]

    async def dispatch(self, request: Request, call_next):
        """Pretty print JSON Response."""
        response = await call_next(request)

        # Skip excluded paths and non-streaming responses
        if request.url.path in self.excluded_paths or not isinstance(response, StreamingResponse):
            return response

        # Collect the stream into a single bytes object
        body = b""
        async for chunk in response.body_iterator:
            if isinstance(chunk, str):
                chunk = chunk.encode()  # Ensure chunk is bytes
            body += chunk

        # Only modify application/json responses
        if response.headers.get("content-type") == "application/json":
            data = json.loads(body.decode())
            pretty_json = json.dumps(
                data,
                ensure_ascii=False,
                allow_nan=False,
                indent=4,
                separators=(", ", ": "),
            ).encode("utf-8")

            # Create a new response with the pretty JSON and original status code
            response.headers["Content-Length"] = str(len(pretty_json))
            return Response(
                content=pretty_json,
                status_code=response.status_code,
                media_type="application/json",
                headers=dict(response.headers),
            )

        return response


class Direction(str, Enum):
    """Altissimo FastAPI Direction class."""
    ASCENDING = "ascending"
    DESCENDING = "descending"


class PaginatedList(BaseModel):
    """Altissimo FastAPI Paginated List class."""
    items: list = []
    next_cursor: str = ""
    previous_cursor: str = ""
    total: Optional[int] = None


class Pagination(BaseModel):
    """Altissimo FastAPI Pagination class."""
    limit: int = Query(default=100)
    next_cursor: str = Query(default="")
    previous_cursor: str = Query(default="")
    order_by: str = Query(default="id")
    direction: Direction = Query(default=Direction.ASCENDING)
    include_total: bool = Query(default=False)

    @model_validator(mode="after")
    def check_cursors(self):
        """Check that only one of next_cursor or prev_cursor is set."""
        if self.next_cursor and self.previous_cursor:
            raise ValueError("Only one of next_cursor or previous_cursor can be set.")
        return self



"""Altissimo SecretManager Tests file."""
from .secretmanager import SecretManager


def test_get_secret():
    """Test the get_secret function."""
    s = SecretManager()
    assert s.get_secret("test-secret", project="altissimo-dev") == "test-secret-value"



"""Altissimo Tools Tests file."""
from .firestore import Firestore

DICT = {'1': {'id': '1', 'name': 'One'}}
LIST = [{'id': '1', 'name': 'One'}]


def test_get_collection():
    """Test get_collection."""
    f = Firestore()
    assert f.get_collection("testing") == LIST


def test_get_collection_dict():
    """Test get_collection."""
    f = Firestore()
    assert f.get_collection_dict("testing") == DICT


def test_get_collection_group():
    """Test get_collection_group."""
    f = Firestore()
    assert f.get_collection_group("testing") == LIST


def test_get_collection_group_dict():
    """Test get_collection_group."""
    f = Firestore()
    assert f.get_collection_group_dict("testing") == DICT


def test_get_docs():
    """Test get_docs."""
    f = Firestore()
    assert f.get_docs("testing")[0].id == "1"
    assert f.get_docs("testing")[0].get("name") == "One"


def test_get_docs_dict():
    """Test get_docs."""
    f = Firestore()
    assert f.get_docs_dict("testing")["1"].id == DICT["1"]["id"]
    assert f.get_docs_dict("testing")["1"].get("name") == DICT["1"]["name"]



"""Altissimo SecretManager Class file."""
import os

from google.cloud import secretmanager_v1

GOOGLE_CLOUD_PROJECT = os.environ.get("GOOGLE_CLOUD_PROJECT")


class SecretManager:
    """Altissimo SecretManager Class."""

    def __init__(self, project=None, credentials=None):
        """Initialize the SecretManager class."""
        self.client = secretmanager_v1.SecretManagerServiceClient(
            credentials=credentials,
        )
        self.project = project or GOOGLE_CLOUD_PROJECT

    def get_name(self, secret, version="latest", project=None):
        """Return the full name of the secret."""
        project = project or self.project
        return f"projects/{project}/secrets/{secret}/versions/{version}"

    def get_secret(self, secret, version="latest", project=None):
        """Return the value of a secret."""
        name = self.get_name(secret, version, project)
        request = secretmanager_v1.AccessSecretVersionRequest(name=name)
        response = self.client.access_secret_version(request=request)
        return response.payload.data.decode("utf-8")



# -*- coding: utf-8 -*-
"""Google Sheets API."""

from googleapiclient.discovery import build

from altissimo.googleapiclient import GoogleAPIClient


class Sheets(GoogleAPIClient):
    """Sheets class."""

    def __init__(self, credentials=None):
        """Initialize a class instance."""
        self.sheets = build(
            "sheets",
            "v4",
            credentials=credentials,
            cache_discovery=False,
        )

    def batchupdate_sheet(self, spreadsheet_id, body):
        """Update a Google sheet."""
        return self.sheets.spreadsheets().batchUpdate(
            spreadsheetId=spreadsheet_id,
            body=body,
        ).execute()

    def clear_sheet(self, spreadsheet_id, range_name="Sheet1!A:Z"):
        """Clear the values from a Google Sheet."""
        return self.sheets.spreadsheets().values().clear(
            spreadsheetId=spreadsheet_id,
            range=range_name,
            body={},
        ).execute()

    def get_sheet(self, spreadsheet_id, range_name="Sheet1!A:Z"):
        """Return the data from a Google Sheet."""
        return self.sheets.spreadsheets().values().get(
            spreadsheetId=spreadsheet_id,
            range=range_name,
        ).execute()

    def update_sheet(
            self,
            spreadsheet_id,
            body,
            range_name="Sheet1!A:Z",
            value_input_option="RAW"
    ):
        """Update a Google sheet."""
        return self.sheets.spreadsheets().values().update(
            spreadsheetId=spreadsheet_id,
            range=range_name,
            body=body,
            valueInputOption=value_input_option
        ).execute()



"""Altissimo API Tests file."""
import jwt
# import pytest
# import werkzeug.exceptions

from .api import API
import altissimo.api


class Request:

    def __init__(self, headers):
        self.headers = headers


audience = "https://audience"
issuer = "auth@altissimo.io"
secret_key = "randomsecretkey"
auth_token = API(secret_key).create_jwt(
    {"user": {}},
    aud=audience,
    iss=issuer,
)
empty_auth_token = jwt.encode({}, secret_key, algorithm="HS256")
dataless_auth_token = API(secret_key).create_jwt({})
expired_auth_token = API(secret_key).create_jwt({"user": {}}, seconds=-3600)
invalid_auth_token = "abcdefg"

auth_header = {"Authorization": f"Bearer {auth_token}"}
bad_auth_header = {"Authorization": "Bacon "}
dataless_auth_header = {"Authorization": f"Bearer {dataless_auth_token}"}
empty_auth_header = {"Authorization": f"Bearer {empty_auth_token}"}
expired_auth_header = {"Authorization": f"Bearer {expired_auth_token}"}
invalid_auth_header = {"Authorization": f"Bearer {invalid_auth_token}"}


def test_authorize_request(mocker):
    """Test authorize_request."""
    a = API(secret_key)
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.authorize_request() is True
    mocked_abort.assert_not_called()


def test_check_auth_header(mocker):
    """Test check_auth_header."""
    a = API()
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == request.headers["Authorization"]
    mocked_abort.assert_not_called()


def test_check_auth_header_missing(mocker):
    """Test check_auth_header with missing header."""
    a = API()
    request = Request({})
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() is None
    mocked_abort.assert_called_once_with(401, "Required Authorization header not found.")


def test_check_auth_header_bearer(mocker):
    """Test check_auth_header_bearer."""
    a = API()
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header()
    assert a.check_auth_header_bearer() is True
    mocked_abort.assert_not_called()


def test_check_auth_header_bearer_missing(mocker):
    """Test check_auth_header_bearer with missing bearer."""
    a = API()
    request = Request(bad_auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header()
    assert a.check_auth_header_bearer() is None
    mocked_abort.assert_called_once_with(401, "Invalid Authorization header. Bearer type required.")


def test_check_auth_token(mocker):
    """Test check_auth_token."""
    a = API()
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header()
    assert a.check_auth_token() == auth_token
    mocked_abort.assert_not_called()


def test_check_auth_token_missing(mocker):
    """Test check_auth_token with missing token."""
    a = API()
    request = Request(bad_auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header()
    assert a.check_auth_token() is None
    mocked_abort.assert_called_once_with(401, "Required Bearer token not found.")


def test_validate_auth_token(mocker):
    """Test validate_auth_token."""
    a = API(secret_key)
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header()
    assert a.check_auth_token() == auth_token
    assert a.validate_auth_token() is True
    mocked_abort.assert_not_called()


def test_validate_auth_token_expired(mocker):
    """Test validate_auth_token for expired token."""
    a = API(secret_key)
    request = Request(expired_auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == expired_auth_header["Authorization"]
    assert a.check_auth_token() == expired_auth_token
    assert a.validate_auth_token() is None
    mocked_abort.assert_called_once_with(401, "Token expired.")


def test_validate_auth_token_invalid(mocker):
    """Test validate_auth_token for invalid token."""
    a = API(secret_key)
    request = Request(invalid_auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == invalid_auth_header["Authorization"]
    assert a.check_auth_token() == invalid_auth_token
    assert a.validate_auth_token() is None
    mocked_abort.assert_called_once_with(401, "Invalid token.")


def test_validate_auth_token_audience(mocker):
    """Test validate_auth_token incorrect audience."""
    a = API(secret_key)
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == auth_header["Authorization"]
    assert a.check_auth_token() == auth_token
    assert a.validate_auth_token(
        audience="http://example.net",
        options={
            "require": ["aud"],
            "verify_aud": True,
        },
    ) is None
    mocked_abort.assert_called_once_with(401, "Invalid audience.")


def test_validate_auth_token_empty(mocker):
    """Test validate_auth_token for no payload."""
    a = API(secret_key)
    request = Request(empty_auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == empty_auth_header["Authorization"]
    assert a.check_auth_token() == empty_auth_token
    assert a.validate_auth_token(options={"require": []}) is None
    mocked_abort.assert_called_once_with(401, "Invalid token. Missing payload.")


def test_validate_auth_token_issuer(mocker):
    """Test validate_auth_token invalid issuer."""
    a = API(secret_key)
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == auth_header["Authorization"]
    assert a.check_auth_token() == auth_token
    assert a.validate_auth_token(
        issuer="invalid-issuer",
        options={
            "require": ["iss"],
            "verify_iss": True,
        },
    ) is None
    mocked_abort.assert_called_once_with(401, "Invalid issuer.")


def test_validate_auth_token_missing_claim(mocker):
    """Test validate_auth_token missingd claim."""
    a = API(secret_key)
    request = Request(auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == auth_header["Authorization"]
    assert a.check_auth_token() == auth_token
    assert a.validate_auth_token(
        options={
            "require": ["sub"],
        },
    ) is None
    mocked_abort.assert_called_once_with(401, "Missing required claim. [Token is missing the \"sub\" claim]")


def test_validate_auth_token_missing_data(mocker):
    """Test validate_auth_token missingd data."""
    a = API(secret_key)
    request = Request(dataless_auth_header)
    mocker.patch.object(altissimo.api, "request", request)
    mocked_abort = mocker.patch.object(altissimo.api, "abort")
    assert a.check_auth_header() == dataless_auth_header["Authorization"]
    assert a.check_auth_token() == dataless_auth_token
    assert a.validate_auth_token() is None
    mocked_abort.assert_called_once_with(401, "Invalid token. Missing data.")



# -*- coding: utf-8 -*-
"""GoogleAPIClient class file."""


class GoogleAPIClient:
    """GoogleAPIClient service class."""

    def get_list_items(self, method, request, name):
        """Return a list of items from a GoogleAPIClient list/list_next cycle."""
        items = []
        while request is not None:
            response = request.execute()
            items += response.get(name, [])
            request = method.list_next(request, response)
        return items

    def get_list_items_iterator(self, method, request, name):
        """Yeild items from a GoogleAPIClient list/list_next cycle."""
        while request is not None:
            response = request.execute()
            items = response.get(name, [])
            for item in items:
                yield item
            request = method.list_next(request, response)



"""Altissimo Sheets Tests file."""
from .sheets import Sheets

SHEET_ID = "1PFcNXkw0Om8oDwIIMEBuJcZIMvMCTWRnL0V8hgUbGyc"

RANGE = 'Sheet1!A1:Z1000'
DATA = {'majorDimension': 'ROWS', 'range': RANGE, 'values': [['Name', 'Test Sheet']]}
EMPTY_DATA = {'majorDimension': 'ROWS', 'range': RANGE}
BATCH_DATA = {"requests": [DATA]}
BATCH_RESPONSE = {"replies": [{}], 'spreadsheetId': SHEET_ID}
CLEAR_RESPONSE = {'clearedRange': 'Sheet1!A1:Z1000', 'spreadsheetId': SHEET_ID}
REQUEST = {
    "sortRange": {
        "range": {
            "sheet_id": 0,
            "start_row_index": 0,
            "end_row_index": 1,
            "start_column_index": 0,
            "end_column_index": 1,
        },
        "sortSpecs": [
            {
                "dimensionIndex": 0,
                "sortOrder": "ASCENDING"
            }
        ]
    },
}
UPDATE_RESPONSE = {
    'updatedRange': 'Sheet1!A1:B1',
    'spreadsheetId': SHEET_ID,
    'updatedCells': 2,
    'updatedColumns': 2,
    'updatedRows': 1,
}


def test_batchupdate_sheet():
    """Test the batchupdate_sheet function."""
    s = Sheets()
    assert s.batchupdate_sheet(SHEET_ID, {"requests": [REQUEST]}) == BATCH_RESPONSE


def test_get_sheet():
    """Test the get_sheet function."""
    s = Sheets()
    assert s.get_sheet(SHEET_ID) == DATA


def test_clear_sheet():
    """Test the clear_sheet function."""
    s = Sheets()
    assert s.clear_sheet(SHEET_ID, RANGE) == CLEAR_RESPONSE
    assert s.get_sheet(SHEET_ID) == EMPTY_DATA
    assert s.update_sheet(SHEET_ID, DATA, RANGE) == UPDATE_RESPONSE


def test_update_sheet():
    """Test the update_sheet function."""
    s = Sheets()
    assert s.update_sheet(SHEET_ID, DATA, RANGE) == UPDATE_RESPONSE



"""Altissimo GoogleAPIClient Tests file."""
from googleapiclient.discovery import build

from .googleapiclient import GoogleAPIClient


def test_get_list_items():
    """Test get_list_items."""
    g = GoogleAPIClient()
    drive = build("drive", "v3", cache_discovery=False)
    files = drive.files()
    request = files.list(pageSize=1)
    response = g.get_list_items(files, request, "files")
    assert len(response) == 2


def test_get_list_items_iterator():
    """Test get_list_items_iterator."""
    g = GoogleAPIClient()
    drive = build("drive", "v3", cache_discovery=False)
    files = drive.files()
    request = files.list(pageSize=1)
    response = list(g.get_list_items_iterator(files, request, "files"))
    assert len(response) == 2
